"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const db_1 = require("../../data-service/db");
const axios_1 = __importDefault(require("axios"));
const lodash_1 = __importDefault(require("lodash"));
const device_service_1 = require("../../data-service/device-service");
const logger_1 = __importDefault(require("../../logger"));
const device_managers_1 = require("../../device-managers");
const typedi_1 = __importDefault(require("typedi"));
const SERVER_UP_TIME = new Date().toISOString();
function getDevices(request, response) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        let devices = (yield db_1.ADTDatabase.DeviceModel).find();
        const { sessionId } = request.query;
        if (sessionId) {
            return response.json(devices.find((value) => value.session_id === sessionId));
        }
        /* dashboard-plugin-url is the base url for opening the appium-dashboard-plugin
         * This value will be attached to all express request via middleware
         */
        const dashboardPluginUrl = request['dashboard-plugin-url'];
        if (dashboardPluginUrl) {
            const sessions = ((_b = (_a = (yield axios_1.default.get(`${dashboardPluginUrl}/api/sessions?start_time=${SERVER_UP_TIME}`)).data) === null || _a === void 0 ? void 0 : _a.result) === null || _b === void 0 ? void 0 : _b.rows) || [];
            const deviceSessionMap = {};
            sessions.forEach((session) => {
                if (!deviceSessionMap[session.udid]) {
                    deviceSessionMap[session.udid] = [];
                }
                deviceSessionMap[session.udid].push(session);
            });
            devices = devices.map((d) => {
                var _a;
                d.dashboard_link = `${dashboardPluginUrl}?device_udid=${d.udid}&start_time=${SERVER_UP_TIME}`;
                d.total_session_count = ((_a = deviceSessionMap[d.udid]) === null || _a === void 0 ? void 0 : _a.length) || 0;
                return d;
            });
        }
        return response.json(devices);
    });
}
function getDeviceByPlatform(request, response) {
    return __awaiter(this, void 0, void 0, function* () {
        const { platform } = request.params;
        const { deviceType, booted } = request.query;
        if (!platform || ['ios', 'android'].indexOf(platform.toLowerCase()) < 0) {
            return response.status(200).send([]);
        }
        let devices = (yield db_1.ADTDatabase.DeviceModel).find({
            platform: platform.toLowerCase(),
        });
        if (!lodash_1.default.isNil(deviceType)) {
            devices = devices.filter((value) => value.deviceType === deviceType);
        }
        if (!lodash_1.default.isNil(booted)) {
            devices = devices.filter((d) => d.state === 'Booted');
        }
        return response.status(200).send(devices);
    });
}
function registerNode(request, response) {
    return __awaiter(this, void 0, void 0, function* () {
        const requestBody = request.body;
        const { type } = request.query;
        if (type === 'add') {
            const addedDevices = yield (0, device_service_1.addNewDevice)(requestBody);
            if (addedDevices.length > 0) {
                logger_1.default.info(`Added new devices: ${JSON.stringify(addedDevices)}`);
            }
        }
        else if (type === 'remove') {
            yield (0, device_service_1.removeDevice)(requestBody);
        }
        response.status(200).send({
            success: true,
        });
    });
}
function blockDevice(request, response) {
    return __awaiter(this, void 0, void 0, function* () {
        const requestBody = request.body;
        const device = yield (0, device_service_1.getDevice)(requestBody);
        if (!lodash_1.default.isNil(device)) {
            yield (0, device_service_1.userBlockDevice)(device.udid, device.host);
        }
        response.status(200).send({
            success: true,
        });
    });
}
function unBlockDevice(request, response) {
    return __awaiter(this, void 0, void 0, function* () {
        const requestBody = request.body;
        const device = yield (0, device_service_1.getDevice)(requestBody);
        if (!lodash_1.default.isNil(device)) {
            yield (0, device_service_1.userUnblockDevice)(device.udid, device.host);
        }
        response.status(200).send({
            success: true,
        });
    });
}
function getQueuedSessionLength(request, response) {
    return __awaiter(this, void 0, void 0, function* () {
        response.json((yield db_1.ADTDatabase.PendingSessionsModel).chain().find().count());
    });
}
function getQueuedSessionRequests(request, response) {
    return __awaiter(this, void 0, void 0, function* () {
        response.json((yield db_1.ADTDatabase.PendingSessionsModel).chain().find().data());
    });
}
function getNodes(request, response) {
    return __awaiter(this, void 0, void 0, function* () {
        // unfortunately, lokijs does not support field projection
        const nodes = yield (yield db_1.ADTDatabase.DeviceModel)
            .chain()
            .find()
            .data()
            .map((node) => {
            // return ony host
            return node.host;
        });
        // unique nodes
        const uniqueNodes = lodash_1.default.uniq(nodes);
        response.json(uniqueNodes);
    });
}
function nodeAdbStatusOnOtherHost(currentHost, request, response) {
    return __awaiter(this, void 0, void 0, function* () {
        const { host } = request.params;
        // when host is this hub, return status from AndroidDeviceManager directly
        // otherwise, forward request to the node
        logger_1.default.info(`currentHost: ${currentHost}, host: ${host}`);
        if (host === currentHost) {
            const devices = yield getDevicesFromDeviceManager();
            response.json(devices.map((device) => {
                return {
                    udid: device.udid,
                    host: device.host,
                    state: device.state,
                    platform: device.platform,
                };
            }));
        }
        else {
            // find node url from database of devices
            const devices = (yield (yield db_1.ADTDatabase.DeviceModel)
                .chain()
                .find({ host: { $contains: host } })
                .data());
            if (devices.length === 0) {
                response
                    .status(404)
                    .send(`Host ${host} does not have any devices listed in database. I don't know how to forward request to that host`);
                return;
            }
            const device = devices[0];
            // if device is a cloud device, return error
            if (device.cloud) {
                response.status(400).send('Getting status from cloud node is not supported');
                return;
            }
            // remove wd/hub from url
            const normalizedUrl = device.host.replace(/\/wd\/hub$/, '');
            const url = `${normalizedUrl}/device-farm/api/node/status`;
            const result = yield axios_1.default.get(url);
            response.json(result.data);
        }
    });
}
function nodeAdbStatusOnThisHost(request, response) {
    return __awaiter(this, void 0, void 0, function* () {
        const devices = yield getDevicesFromDeviceManager();
        // return udid, host, state
        response.json(devices.map((device) => {
            return {
                udid: device.udid,
                host: device.host,
                state: device.state,
                platform: device.platform,
            };
        }));
    });
}
/**
 * Returns all devices from all device managers (this host only)
 * @returns IDevice[]
 */
function getDevicesFromDeviceManager() {
    return __awaiter(this, void 0, void 0, function* () {
        const dfm = typedi_1.default.get(device_managers_1.DeviceFarmManager);
        const instances = yield dfm.deviceInstances();
        // return devices from all device managers
        const devices = [];
        for (const instance of instances) {
            const instanceDevices = yield instance.getDevices({
                androidDeviceType: 'both',
                iosDeviceType: 'both',
            }, []);
            devices.push(...instanceDevices);
        }
        return devices;
    });
}
function register(router, pluginArgs) {
    router.get('/device', getDevices);
    router.get('/device/:platform', getDeviceByPlatform);
    router.post('/register', registerNode);
    router.post('/block', blockDevice);
    router.post('/unblock', unBlockDevice);
    // session related
    router.get('/queue/length', getQueuedSessionLength);
    router.get('/queue', getQueuedSessionRequests);
    // node related routes
    router.get('/node', getNodes);
    router.get('/node/status', nodeAdbStatusOnThisHost);
    router.get('/node/:host/status', lodash_1.default.curry(nodeAdbStatusOnOtherHost)(pluginArgs.bindHostOrIp));
    // node status
    router.get('/status', (request, response) => {
        response.json({
            status: 'ok',
            version: process.env.npm_package_version || 'unknown (not running from npm package)',
        });
    });
}
exports.default = {
    register,
};
